import requests

from . import idict


def __init__(params: dict):
    return IsddcAPI(**params)


class IsddcAPI:
    headers = {
        'Content-Type': 'application/json',
        'X-Auth-Token': None
    }

    uri = idict(
        metrics='/v2.0/metrics',
        inventories='/v2.0/inventories'
    )

    def __init__(
            self,
            api: str,
            keystone: str,
            username: str,
            password: str,
            tenantid: str,
            timeout: int = 30
    ):
        self.api = api
        self.keystone = keystone
        self.username = username
        self.password = password
        self.tenantid = tenantid
        self.timeout = timeout

        self.auth_api = f'http://{self.keystone}/v3/auth/tokens'

        for name, uri in self.uri.items():
            self.uri[name] = f'http://{self.api}{uri}'

    def __getattribute__(self, name):
        if name == 'send':
            self.headers['X-Auth-Token'] = self._x_auth_token

        return super().__getattribute__(name)

    @property
    def _x_auth_token(self) -> str:
        return requests.post(
            url=self.auth_api,
            json={'auth': {
                'identity': {
                    'methods': ['password'],
                    'password': {'user': {
                        'domain': {'id': 'default'},
                        'name': self.username,
                        'password': self.password}}},
                'scope': {'project': {
                    'id': self.tenantid,
                    'domain': {'id': 'default'}}}}}
        ).headers['X-Subject-Token']

    def send(
            self,
            data: dict,
            dtype: 'enum(metrics, inventories)'
    ) -> requests.Response:
        return requests.post(
            url=self.uri[dtype],
            json=data,
            headers=self.headers,
            timeout=self.timeout
        )
